<?php

namespace FiloBlu\CheckoutExtend\Block\Plugin\Checkout;

class LayoutProcessor {

    public $_helper;
    protected $_filterProvider;
    protected $_storeManager;
    protected $_blockFactory;
    protected $_customerSession;
    protected $_subscriberFactory;

    public function __construct(\FiloBlu\CheckoutExtend\Helper\Data $helper, \Magento\Cms\Model\Template\FilterProvider $filterProvider, \Magento\Store\Model\StoreManagerInterface $storeManager, \Magento\Cms\Model\BlockFactory $blockFactory, \Magento\Customer\Model\Session $customerSession, \Magento\Newsletter\Model\SubscriberFactory $subscriberFactory) {
        $this->_helper = $helper;
        $this->_filterProvider = $filterProvider;
        $this->_storeManager = $storeManager;
        $this->_blockFactory = $blockFactory;
        $this->_customerSession = $customerSession;
        $this->_subscriberFactory = $subscriberFactory;
    }

    public function aroundProcess($subject, \Closure $proceed, $jsLayout) {

        $storeId = $this->_storeManager->getStore()->getId();

        # Shipping Additional Data
        $_shippingAdditionalData = $this->_helper->getShippingAddressAdditionalData();

        if ($_shippingAdditionalData && count($_shippingAdditionalData)) {

            foreach ($_shippingAdditionalData as $_additionalData) {
                if ($_additionalData['data_region'] == "tooltip") {
                    $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']
                            ['children']['shippingAddress']['children']['shipping-address-fieldset']
                            ['children'][$_additionalData['input_code']]['config'][$_additionalData['data_region']]["description"] = $_additionalData['label'];
                } else {
                    $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']
                            ['children']['shippingAddress']['children']['shipping-address-fieldset']
                            ['children'][$_additionalData['input_code']]['config'][$_additionalData['data_region']] = $_additionalData['label'];
                }
            }
        }

        # Move Coupon Into Totals
        if ($this->_helper->getConfigValue("filoblu_checkoutextend_section/totals/move_coupon_into_totals")) {
            $jsLayout['components']['checkout']['children']['steps']['children']['billing-step']
                    ['children']['payment']['children']['afterMethods']
                    ['children']['discount']['config']["componentDisabled"] = true;
            $jsLayout['components']['checkout']['children']['sidebar']['children']['summary']
                    ['children']['itemsBefore']['children']['discount'] = [
                'component' => 'FiloBlu_CheckoutExtend/js/view/payment/discount',
                'sortOrder' => 0,
                'children' => [
                    'errors' => [
                        'component' => 'Magento_SalesRule/js/view/payment/discount-messages',
                        'sortOrder' => 0,
                        'displayArea' => 'messages',
                    ],
                ],
            ];
        }

        if ($this->_helper->getConfigValue("filoblu_checkoutextend_section/newsletter/enabled") && !$this->customerAlreadySubscribed()) {


            $cmsBlockCheckoutNewsletterSubscription = $this->_blockFactory->create();
            $cmsBlockCheckoutNewsletterSubscription->setStoreId($storeId)->load("checkout_newsletter_subscription");
            $cmsBlockCheckoutNewsletterSubscriptionHtml = "";

            if ($cmsBlockCheckoutNewsletterSubscription->isActive()) {
                $cmsBlockCheckoutNewsletterSubscriptionHtml = $this->_filterProvider->getBlockFilter()->setStoreId($storeId)->filter($cmsBlockCheckoutNewsletterSubscription->getContent());
            }

            $is_subscribe_for_newsletter = [
                'component' => 'Magento_Ui/js/form/element/boolean',
                'config' => [
                    // customScope is used to group elements within a single form (e.g. they can be validated separately)
                    'customScope' => 'shippingAddress.custom_attributes',
                    'customEntry' => null,
                    'template' => 'ui/form/field',
                    'elementTmpl' => 'FiloBlu_CheckoutExtend/ui/form/element/newsletter_checkbox',
                ],
                'dataScope' => 'shippingAddress.custom_attributes.is_subscribe_for_newsletter',
                'label' => __('Sign up to our newsletter'),
                'description' => $cmsBlockCheckoutNewsletterSubscriptionHtml,
                'provider' => 'checkoutProvider',
                'sortOrder' => 1,
                'options' => [],
                'filterBy' => null,
                'customEntry' => null,
                'visible' => true,
            ];

//            $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['before-fields']['children']['is_subscribe_for_newsletter'] = $is_subscribe_for_newsletter;
            $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['shippingAddress']['children']['before-form']['children']['is_subscribe_for_newsletter'] = $is_subscribe_for_newsletter;
//            $jsLayout['components']['checkout']['children']['steps']['children']['shipping-step']['children']['is_subscribe_for_newsletter'] = $is_subscribe_for_newsletter;
        }
        
        $customJsLayout = $proceed($jsLayout);

        return $customJsLayout;
    }
    
    private function customerAlreadySubscribed() {

        $isSubscribed = false;

        if ($this->_customerSession->isLoggedIn()) {
            $email = $this->_customerSession->getCustomerDataObject()->getEmail();

            $subscriber = $this->_subscriberFactory->create()->loadByEmail($email);

            if ($subscriber->getSubscriberStatus() == \Magento\Newsletter\Model\Subscriber::STATUS_SUBSCRIBED) {
                $isSubscribed = true;
            }
        }

        return $isSubscribed;
    }

}
